@@ -1,3 +1,4 @@
+import mimetypes
 import os
 import re
 import requests
@@ -23,7 +24,7 @@ class ThumborStorageFile(ImageFile):
 
         url = "%s/image" % settings.THUMBOR_WRITABLE_SERVER
         headers = {
-            "Content-Type": "image/jpeg",
+            "Content-Type": mimetypes.guess_type(self.name)[0] or "image/jpeg",
             "Slug": self.name,
         }
         response = requests.post(url, data=image_content, headers=headers)
@@ -45,6 +46,11 @@ class ThumborStorageFile(ImageFile):
 
     file = property(_get_file, _set_file)
     
+    @property
+    def size(self):
+        self.seek(0, os.SEEK_END)
+        return self.tell()
+
 
 class ThumborStorage(Storage):
     """Thumbor Simple Storage Service"""
@@ -78,8 +84,8 @@ class ThumborStorage(Storage):
             return False
 
     def size(self, name):
-        pass
-        # https://docs.djangoproject.com/en/1.5/ref/files/storage/#django.core.files.storage.Storage.size
+        f = self.open(name)
+        return f.size
 
     def url(self, name):
         return thumbor_image_url(name)
@@ -122,10 +128,12 @@ class ThumborMigrationStorage(ThumborStorage, FileSystemStorage):
         else:
             return FileSystemStorage.url(self, name)
 
+    def path(self, name):
+        if re.match(r"^/image/\w{32}/.*$", name):
+            return ThumborStorage.path(self, name)
+        else:
+            return FileSystemStorage.path(self, name)
 
-## These functions because some methods in ThumborStorage may be called with
-#   self being a ThumborMigrationStorage instance and result in infinite loop.
-# These methods proxiing to these functions.
 
 def thumbor_original_exists(url):
     # May be cool to be able to check if the image exists on Thumbor server
@@ -142,5 +150,9 @@ def thumbor_original_exists(url):
         return False
 
 
+## These functions because some methods in ThumborStorage may be called with
+#   self being a ThumborMigrationStorage instance and result in infinite loop.
+# These methods proxiing to these functions.
+
 def thumbor_image_url(name):
     return "%s%s" % (settings.THUMBOR_SERVER, name)
